# TranslationBliss

TranslationBliss is a translation suite optimized for site owners.

## How to use

After installation, TranslationBliss offers a highly efficient workflow to get any multilingual site into production.
Assuming a en / de / fr site:
- Run `drush potxx` to extract all translatable strings and current translations to .po files.
- Translate the .po files.
- Re-import via `drush locale-import`

The main benefits over core config_translation and potx modules:
- Difference to potx module...
  - he .po files include translatable strings from all config, beneath translatable strings from any module in modules/custom/.
  - there is a hook for custom enhancements.
  - default parameters are optimized for site owners.
- Difference to core config translation...
  - all config gets the interface translations applied (not only "shipped" config as core does).
  - there is no need for managing config overrides to get the translations (so the language config overrides in config/sync/language/foo can be empty).
  - the config overrides **can** be leveraged to get differing translations for individual config items though.

Also there are benefits for config with non-english source-language:
- Core has no concept of non-english source language, and treats all non-english config as untranslatable.
- Assume you create a monolingual site, where all config is "langcode = de".
- With core alone, you must manually create translation config overrides for every config.
- OR, you "feature" or "config_devel" the config to sth. like `modules/custom/my_custom_config/config/install` (effectively making it "shipped" in core terms) AND change all config to be english source language. Then core will sync all your config translations with interface translations, with no exceptions.
- With TranslationBliss though, all translatable "de" config strings are available for translation (marked with a `[srclang=de]` context prefix, either in .po file or in the UI). Note that core already allows you to switch on translate-to-english.

## Technical background

TranslationBliss contains:
- Config Translation Bliss
- SourceLanguage API
- Translation Extraction Bliss (PotXX)

## Config Translation Bliss

A different approach to config translation that is more suited for site owners.
Comparison:

|                                                        | Config Translation Bliss | Core |
|--------------------------------------------------------| ------------------------ | ---- |
| How it's done                                          | Translatable strings from ALL config are translated on the fly like any interface string, EXCEPT when there is an explicit config translation (language override) from the config_translation module. Nothing is synced between these. | Interface translation are synced back and forth with translations in config language overrides, but only for module-owned config (i.e. from any module/foo/config/install). Which means: Changing any interface string translation will automagically update all language config overrides. Adjusting config translation for any module-owned config will change interface translation for that string, which in turn will change translation of all other module-owned config.|
| Differing translations of one string                   | Possible via explicit config translation language overrides. Site builder has full control over each config's individual translaton, i.e. effectively one context per config. | A unique source string is enforced to have one and only one unique translation across code and config. |
| Source language                                        |Non-english default config is available to Template Extraction (PotX) and interface translation UI with a context prefix of (e.g.) "[srclang=de]". | Non-english default config is ignored. |
| Contents of config/sync/language/de/*                  | Only a few config translations, where the default interface translation does not fit. In many cases: none. | For module-owned config: Lots of config overrides that are fully controlled by config translation. For site-onwed config: Site owner provided overrides that have no connection to interface translation. And no practival way to discern them in commits. |
| Effect for module-owned config                         | Same for all config: Interface translation, but only if no explicit config translation. Same result as core, with the additional possibility to translate any config differently. | Translated via interface translation. Config overrides are fully controlled by that. |
| Effect for site-owned config (not from config/install) | Same for all config: Interface translation, but only if no explicit config translation. So site owner benefits from all interface translation done, with the additional possibility to translate any config differently. | Site owner must translate all such config manually. |
| Effect of "featuring" config                           | All config is treated the same. So featuring (writing to a module's config/install via features or config_devel) does not change anything.|Featuring to config/install changes the config's status from site-owned to module-owned, changing its behavior wrt translations. |


## SourceLanguage API

Third party code can leverage the source language API like this:
```php
use Drupal\translation_bliss\SourceLanguage\SourceLanguageTranslationServiceFactoryInterface;

# Adds the [srclang=de] context prefix to all translations.
\Drupal::service(SourceLanguageTranslationServiceFactoryInterface::class)
  ->forSourceLangcode('de')
  ->translate('Sauerkraut');
```

## Translation Extraction Bliss (PotXX)

The command `drush potxx` does translation template extraction optimized for site owners:
- Extracts all code translatables from custom modules (defaults to modules/custom/*).
- Extracts all translatables from all config of the site (active config, not config/sync).
- Extracts template to ../translations/custom/general.potx and current translatons to ../translations/custom/general.de.po (ot other langcode) by default.
- Has drush hookx for custom extensions (for an example see TranslationBlissConfigPotxCommands).
