<?php

declare(strict_types=1);
namespace Drupal\Tests\translation_bliss\Functional;

use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\translation_bliss\Traits\EntityTrait;
use Drupal\Tests\translation_bliss\Traits\FixDriverTrait;
use Drupal\user\Entity\Role;

final class ConfigLanguageSelectorTest extends BrowserTestBase {

  use FixDriverTrait;
  use EntityTrait;

  protected static $modules = [
    'translation_bliss_config_language_select',
    'language',
    // Needed to alter logging form.
    'dblog',
  ];

  protected $defaultTheme = 'stark';

  public function testConfigEntityLanguageSelector() {
    ConfigurableLanguage::createFromLangcode('de')->save();
    ConfigurableLanguage::createFromLangcode('es')->save();
    $this->drupalLogin($this->rootUser);

    // Add role, set 'de'.
    $this->drupalGet('/admin/people/roles/add');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->optionExists('langcode', 'en');
    $this->assertSession()->optionExists('langcode', 'de');
    $this->assertSession()->optionExists('langcode', 'es');

    $this->submitForm([
      'id' => 'test_role',
      'label' => 'Test role',
      'langcode' => 'de',
    ], 'Save');
    $this->assertSession()->statusCodeEquals(200);

    // Verify.
    $role = Role::load('test_role');
    $this->assertSame('de', $role->language()->getId());

    // Edit role, set 'es'.
    $this->drupalGet('/admin/people/roles/manage/test_role');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->optionExists('langcode', 'en');
    $this->assertSession()->optionExists('langcode', 'de');
    $this->assertSession()->optionExists('langcode', 'es');

    $this->submitForm([
      'id' => 'test_role',
      'label' => 'Test role',
      'langcode' => 'es',
    ], 'Save');
    $this->assertSession()->statusCodeEquals(200);

    // Verify.
    $role = $this->reloadEntity($role);
    $this->assertSame('es', $role->language()->getId());

    // Delete role.
    $this->drupalGet('/admin/people/roles/manage/test_role/delete');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->fieldNotExists('langcode');
  }

  public function testConfigSingleFormLanguageSelector() {
    ConfigurableLanguage::createFromLangcode('de')->save();
    ConfigurableLanguage::createFromLangcode('es')->save();
    $this->drupalLogin($this->rootUser);

    // For maintenance code config, set 'de'.
    $this->drupalGet('/admin/config/development/maintenance');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->optionExists('langcode__system__maintenance', 'en');
    $this->assertSession()->optionExists('langcode__system__maintenance', 'de');
    $this->assertSession()->optionExists('langcode__system__maintenance', 'es');

    $this->submitForm([
      'langcode__system__maintenance' => 'de',
    ], 'Save');
    $this->assertSession()->statusCodeEquals(200);

    // Verify.
    $config = $this->config('system.maintenance');
    $this->assertSame('de', $config->get('langcode'));
  }

  public function testConfigMultiFormLanguageSelector() {
    ConfigurableLanguage::createFromLangcode('de')->save();
    ConfigurableLanguage::createFromLangcode('es')->save();
    $this->drupalLogin($this->rootUser);

    // For multi config form, set 'de' / 'es'.
    $this->drupalGet('/admin/config/development/logging');
    $this->assertSession()->statusCodeEquals(200);

    $this->assertSession()->optionExists('langcode__system__logging', 'en');
    $this->assertSession()->optionExists('langcode__system__logging', 'de');
    $this->assertSession()->optionExists('langcode__system__logging', 'es');

    $this->assertSession()->optionExists('langcode__dblog__settings', 'en');
    $this->assertSession()->optionExists('langcode__dblog__settings', 'de');
    $this->assertSession()->optionExists('langcode__dblog__settings', 'es');

    $this->submitForm([
      'langcode__system__logging' => 'de',
      'langcode__dblog__settings' => 'es',
    ], 'Save');
    $this->assertSession()->statusCodeEquals(200);

    // Verify.
    $config = $this->config('system.logging');
    $this->assertSame('de', $config->get('langcode'));

    $config = $this->config('dblog.settings');
    $this->assertSame('es', $config->get('langcode'));
  }

}
